package xyz.zhouxy.plusone.commons.security;

import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;

import javax.crypto.Cipher;

import com.google.common.annotations.Beta;

/**
 * RSA 非对称加密。
 */
@Beta
public class RSA {
    private static final int DEFAULT_KEY_SIZE = 2048;
    public static final String NAME = "RSA";
    private static final String TRANSFORMATION = "RSA/ECB/OAEPWithSHA-256AndMGF1Padding";

    private final PublicKey publicKey;
    private final PrivateKey privateKey;

    private RSA(final PublicKey publicKey, final PrivateKey privateKey) {
        this.publicKey = publicKey;
        this.privateKey = privateKey;
    }

    public static RSA of(final PublicKey publicKey, final PrivateKey privateKey) {
        return new RSA(publicKey, privateKey);

    }

    public static RSA of(final KeyPair keyPair) {
        return new RSA(keyPair.getPublic(), keyPair.getPrivate());

    }

    public static RSA withKeySize(int keySize) {
        try {
            KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance(NAME);
            keyPairGenerator.initialize(keySize);
            final KeyPair keyPair = keyPairGenerator.generateKeyPair();
            return new RSA(keyPair.getPublic(), keyPair.getPrivate());
        } catch (NoSuchAlgorithmException e) {
            throw new SecurityException(e);
        }
    }

    public static RSA withDefaultKeySize() {
        return withKeySize(DEFAULT_KEY_SIZE);
    }

    public PrivateKey getPrivateKey() {
        return privateKey;
    }

    public PublicKey getPublicKey() {
        return publicKey;
    }

    public byte[] encrypt(byte[] input) throws GeneralSecurityException {
        Cipher encryptModeCipher = Cipher.getInstance(TRANSFORMATION);
        encryptModeCipher.init(Cipher.ENCRYPT_MODE, publicKey);
        return encryptModeCipher.doFinal(input);
    }

    public byte[] encrypt(String input) throws GeneralSecurityException {
        return encrypt(input.getBytes(StandardCharsets.UTF_8));
    }

    public String encryptToString(String input) throws GeneralSecurityException {
        return new BigInteger(1, encrypt(input)).toString(16);
    }

    public byte[] decrypt(byte[] input) throws GeneralSecurityException {
        Cipher decryptModeCipher = Cipher.getInstance(TRANSFORMATION);
        decryptModeCipher.init(Cipher.DECRYPT_MODE, privateKey);
        return decryptModeCipher.doFinal(input);
    }

    public byte[] decrypt(String input) throws GeneralSecurityException {
        return decrypt(new BigInteger(input, 16).toByteArray());
    }

    public String decryptToString(String input) throws GeneralSecurityException {
        return new String(decrypt(input));
    }
}
